@@ -63,11 +63,6 @@ FileProcessor::~FileProcessor()
         compressor_ = nullptr;
     }
 
-    for (auto& file : active_files_)
-    {
-        util::platform::FileClose(file.second.fd);
-    }
-
     DecodeAllocator::DestroyInstance();
 }
 
@@ -81,11 +76,10 @@ void FileProcessor::WaitDecodersIdle()
 
 bool FileProcessor::Initialize(const std::string& filename)
 {
-    bool success = OpenFile(filename);
+    bool success = SetActiveFile(filename, true);
 
     if (success)
     {
-        success = SetActiveFile(filename, true);
         success = success && ProcessFileHeader();
     }
     else
@@ -113,51 +107,33 @@ std::string FileProcessor::ApplyAbsolutePath(const std::string& file)
     return absolute_path_ + file;
 }
 
-bool FileProcessor::OpenFile(const std::string& filename)
+bool FileProcessor::ProcessNextFrame()
 {
-    if (active_files_.find(filename) == active_files_.end())
-    {
-        FILE* fd;
-        int   result = util::platform::FileOpen(&fd, filename.c_str(), "rb");
-        if (result || fd == nullptr)
-        {
-            GFXRECON_LOG_ERROR("Failed to open file %s", filename.c_str());
-            error_state_ = kErrorOpeningFile;
-            return false;
-        }
-        else
-        {
-            active_files_.emplace(std::piecewise_construct, std::forward_as_tuple(filename), std::forward_as_tuple(fd));
-            error_state_ = kErrorNone;
-        }
-    }
-
-    return true;
+    auto block_processor = [this]() { return this->ProcessBlocksOneFrame(); };
+    return DoProcessNextFrame(block_processor);
 }
 
-bool FileProcessor::ProcessNextFrame()
+bool FileProcessor::ProcessBlocksOneFrame()
+{
+    for (ApiDecoder* decoder : decoders_)
+    {
+        decoder->SetCurrentFrameNumber(current_frame_number_);
+    }
+    return ProcessBlocks();
+}
+
+bool FileProcessor::DoProcessNextFrame(const std::function<bool()>& block_processor)
 {
     bool success = IsFileValid();
 
     if (success)
     {
-        for (ApiDecoder* decoder : decoders_)
-        {
-            decoder->SetCurrentFrameNumber(current_frame_number_);
-        }
-        success = ProcessBlocks();
+
+        success = block_processor();
     }
     else
     {
-        // If not EOF, determine reason for invalid state.
-        if (GetFileDescriptor() == nullptr)
-        {
-            error_state_ = kErrorInvalidFileDescriptor;
-        }
-        else if (ferror(GetFileDescriptor()))
-        {
-            error_state_ = kErrorReadingFile;
-        }
+        error_state_ = CheckFileStatus();
     }
 
     return success;
@@ -218,7 +194,7 @@ bool FileProcessor::ProcessFileHeader()
     bool               success = false;
     format::FileHeader file_header{};
 
-    ActiveFiles& active_file = active_files_[file_stack_.front().filename];
+    assert(file_stack_.front().active_file);
 
     if (ReadBytes(&file_header, sizeof(file_header)))
     {
@@ -314,7 +290,12 @@ bool FileProcessor::ProcessBlocks()
 
             if (success)
             {
-                if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kFunctionCallBlock)
+                if (SkipBlockProcessing())
+                {
+                    GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, block_header.size);
+                    success = SkipBytes(static_cast<size_t>(block_header.size));
+                }
+                else if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kFunctionCallBlock)
                 {
                     format::ApiCallId api_call_id = format::ApiCallId::ApiCall_Unknown;
 
@@ -448,29 +429,7 @@ bool FileProcessor::ProcessBlocks()
             }
             else
             {
-                if (!feof(GetFileDescriptor()))
-                {
-                    // No data has been read for the current block, so we don't use 'HandleBlockReadError' here, as it
-                    // assumes that the block header has been successfully read and will print an incomplete block at
-                    // end of file warning when the file is at EOF without an error. For this case (the normal EOF case)
-                    // we print nothing at EOF, or print an error message and set the error code directly when not at
-                    // EOF.
-                    GFXRECON_LOG_ERROR("Failed to read block header (frame %u block %" PRIu64 ")",
-                                       current_frame_number_,
-                                       block_index_);
-                    error_state_ = kErrorReadingBlockHeader;
-                }
-                else
-                {
-                    assert(!file_stack_.empty());
-
-                    ActiveFileContext& current_file = GetCurrentFile();
-                    if (current_file.execute_till_eof)
-                    {
-                        file_stack_.pop_back();
-                        success = !file_stack_.empty();
-                    }
-                }
+                success = HandleBlockEof("read", true);
             }
         }
         ++block_index_;
@@ -512,12 +471,18 @@ bool FileProcessor::ReadCompressedParameterBuffer(size_t  compressed_buffer_size
     // This should only be null if initialization failed.
     assert(compressor_ != nullptr);
 
+    const uint8_t* compressed_data = nullptr;
+
     if (compressed_buffer_size > compressed_parameter_buffer_.size())
     {
         compressed_parameter_buffer_.resize(compressed_buffer_size);
     }
-
     if (ReadBytes(compressed_parameter_buffer_.data(), compressed_buffer_size))
+    {
+        compressed_data = compressed_parameter_buffer_.data();
+    }
+
+    if (compressed_data)
     {
         if (parameter_buffer_.size() < expected_uncompressed_size)
         {
@@ -525,7 +490,7 @@ bool FileProcessor::ReadCompressedParameterBuffer(size_t  compressed_buffer_size
         }
 
         size_t uncompressed_size = compressor_->Decompress(
-            compressed_buffer_size, compressed_parameter_buffer_, expected_uncompressed_size, &parameter_buffer_);
+            compressed_buffer_size, compressed_data, expected_uncompressed_size, &parameter_buffer_);
         if ((0 < uncompressed_size) && (uncompressed_size == expected_uncompressed_size))
         {
             *uncompressed_buffer_size = uncompressed_size;
@@ -537,10 +502,12 @@ bool FileProcessor::ReadCompressedParameterBuffer(size_t  compressed_buffer_size
 
 bool FileProcessor::ReadBytes(void* buffer, size_t buffer_size)
 {
-    auto file_entry = active_files_.find(file_stack_.back().filename);
-    assert(file_entry != active_files_.end());
+    // File entry is non-const to allow read bytes to be non-const (i.e. potentially reflect a stateful operation)
+    // without forcing use of mutability
+    const auto& active_file = file_stack_.back().active_file;
+    GFXRECON_ASSERT(active_file);
 
-    if (util::platform::FileRead(buffer, buffer_size, file_entry->second.fd))
+    if (active_file->ReadBytes(buffer, buffer_size))
     {
         bytes_read_ += buffer_size;
         return true;
@@ -550,10 +517,10 @@ bool FileProcessor::ReadBytes(void* buffer, size_t buffer_size)
 
 bool FileProcessor::SkipBytes(size_t skip_size)
 {
-    auto file_entry = active_files_.find(file_stack_.back().filename);
-    assert(file_entry != active_files_.end());
+    const auto& active_file = file_stack_.back().active_file;
+    GFXRECON_ASSERT(active_file);
 
-    bool success = util::platform::FileSeek(file_entry->second.fd, skip_size, util::platform::FileSeekCurrent);
+    bool success = active_file->FileSeek(skip_size, util::platform::FileSeekCurrent);
 
     if (success)
     {
@@ -564,12 +531,13 @@ bool FileProcessor::SkipBytes(size_t skip_size)
     return success;
 }
 
-bool FileProcessor::SeekActiveFile(const std::string& filename, int64_t offset, util::platform::FileSeekOrigin origin)
+bool FileProcessor::SeekActiveFile(const FileInputStreamPtr&      active_file,
+                                   int64_t                        offset,
+                                   util::platform::FileSeekOrigin origin)
 {
-    auto file_entry = active_files_.find(file_stack_.back().filename);
-    assert(file_entry != active_files_.end());
+    GFXRECON_ASSERT(active_file);
 
-    bool success = util::platform::FileSeek(file_entry->second.fd, offset, origin);
+    bool success = active_file->FileSeek(offset, origin);
 
     if (success && origin == util::platform::FileSeekCurrent)
     {
@@ -582,20 +550,46 @@ bool FileProcessor::SeekActiveFile(const std::string& filename, int64_t offset,
 
 bool FileProcessor::SeekActiveFile(int64_t offset, util::platform::FileSeekOrigin origin)
 {
-    return SeekActiveFile(file_stack_.back().filename, offset, origin);
+    return SeekActiveFile(file_stack_.back().active_file, offset, origin);
 }
 
 bool FileProcessor::SetActiveFile(const std::string& filename, bool execute_till_eof)
 {
-    if (active_files_.find(filename) != active_files_.end())
+
+    // Look for the name stream in the cache
+    auto cached_stream = stream_cache_.Lookup(filename);
+
+    FileInputStreamPtr active_file;
+    if (cached_stream.has_value())
     {
-        file_stack_.emplace_back(filename, execute_till_eof);
-        return true;
+        active_file = std::move(*cached_stream);
+
+        // Only valid streams in the cache
+        GFXRECON_ASSERT(active_file);
+        GFXRECON_ASSERT(active_file->IsOpen());
     }
     else
     {
-        return false;
+        // No stream in cache, create one
+        active_file = std::make_shared<FileInputStream>();
+        bool opened = active_file->Open(filename);
+
+        if (!opened || !active_file->IsOpen())
+        {
+            GFXRECON_LOG_ERROR("Failed to open file %s", filename.c_str());
+            error_state_ = kErrorOpeningFile;
+            return false;
+        }
+
+        // It's possible we'll want to use the input streams more than once, (kExecuteBlocksFromFile, usage often
+        // does in test cases), so we'll stash off the stream's shared pointer to a cache
+        stream_cache_.Insert(active_file);
     }
+
+    // Now that we have a new stream or old, push it on the stack
+    file_stack_.emplace_back(std::move(active_file), execute_till_eof);
+    error_state_ = kErrorNone;
+    return true;
 }
 
 bool FileProcessor::SetActiveFile(const std::string&             filename,
@@ -603,10 +597,10 @@ bool FileProcessor::SetActiveFile(const std::string&             filename,
                                   util::platform::FileSeekOrigin origin,
                                   bool                           execute_till_eof)
 {
-    if (active_files_.find(filename) != active_files_.end())
+    bool success = SetActiveFile(filename, execute_till_eof);
+    if (success)
     {
-        file_stack_.emplace_back(filename, execute_till_eof);
-        return SeekActiveFile(filename, offset, origin);
+        return SeekActiveFile(file_stack_.back().active_file, offset, origin);
     }
     else
     {
@@ -616,11 +610,11 @@ bool FileProcessor::SetActiveFile(const std::string&             filename,
 
 void FileProcessor::HandleBlockReadError(Error error_code, const char* error_message)
 {
-    auto file_entry = active_files_.find(file_stack_.back().filename);
-    assert(file_entry != active_files_.end());
+    GFXRECON_ASSERT(!file_stack_.empty());
+    const auto& active_file = file_stack_.back().active_file;
 
     // Report incomplete block at end of file as a warning, other I/O errors as an error.
-    if (feof(file_entry->second.fd) && !ferror(file_entry->second.fd))
+    if (active_file->IsEof() && !active_file->IsError())
     {
         GFXRECON_LOG_WARNING("Incomplete block at end of file");
     }
@@ -874,7 +868,7 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
         format::FillMemoryResourceValueCommandHeader header;
 
         success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
-        success = ReadBytes(&header.resource_value_count, sizeof(header.resource_value_count));
+        success = success && ReadBytes(&header.resource_value_count, sizeof(header.resource_value_count));
 
         if (success)
         {
@@ -2134,13 +2128,14 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
                 std::string filename = util::filepath::Join(absolute_path_, filename_c_str);
 
                 // Check for self references
-                if (!filename.compare(file_stack_.back().filename))
+                if (!filename.compare(file_stack_.back().active_file->GetFilename()))
                 {
                     GFXRECON_LOG_WARNING(
                         "ExecuteBlocksFromFile is referencing itself. Probably this is not intentional.");
                 }
 
-                success = OpenFile(filename);
+                success = SetActiveFile(
+                    filename, exec_from_file.offset, util::platform::FileSeekSet, exec_from_file.n_blocks == 0);
                 if (success)
                 {
                     for (auto decoder : decoders_)
@@ -2149,8 +2144,6 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
                             exec_from_file.thread_id, exec_from_file.n_blocks, exec_from_file.offset, filename);
                     }
 
-                    SetActiveFile(
-                        filename, exec_from_file.offset, util::platform::FileSeekSet, exec_from_file.n_blocks == 0);
                     // We need to add 1 because it will be decremented right after this function returns
                     file_stack_.back().remaining_commands = exec_from_file.n_blocks + 1;
                 }
@@ -2257,8 +2250,8 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
     }
     else
     {
-        if ((meta_data_type == format::MetaDataType::kReserved23) ||
-            (meta_data_type == format::MetaDataType::kReserved25))
+        if (meta_data_type >= format::MetaDataType::kBeginExperimentalReservedRange ||
+            meta_data_type == format::MetaDataType::kReserved23 || meta_data_type == format::MetaDataType::kReserved25)
         {
             // Only log a warning once if the capture file contains blocks that are a "reserved" meta data type.
             GFXRECON_LOG_WARNING_ONCE("This capture file contains meta-data block(s) with reserved type(s) that are "
@@ -2456,5 +2449,44 @@ void FileProcessor::PrintBlockInfo() const
     }
 }
 
+bool FileProcessor::HandleBlockEof(const char* operation, bool report_frame_and_block)
+{
+
+    bool success = false;
+    if (!AtEof())
+    {
+        // No data has been read for the current block, so we don't use 'HandleBlockReadError' here, as it
+        // assumes that the block header has been successfully read and will print an incomplete block at
+        // end of file warning when the file is at EOF without an error. For this case (the normal EOF case)
+        // we print nothing at EOF, or print an error message and set the error code directly when not at
+        // EOF.
+        if (report_frame_and_block)
+        {
+            GFXRECON_LOG_ERROR("Failed to %s block header (frame %u block %" PRIu64 ")",
+                               operation,
+                               current_frame_number_,
+                               block_index_);
+        }
+        else
+        {
+            GFXRECON_LOG_ERROR("Failed to %s block header", operation);
+        }
+
+        error_state_ = kErrorReadingBlockHeader;
+    }
+    else
+    {
+        GFXRECON_ASSERT(!file_stack_.empty());
+
+        ActiveFileContext& current_file = GetCurrentFile();
+        if (current_file.execute_till_eof)
+        {
+            file_stack_.pop_back();
+            success = !file_stack_.empty();
+        }
+    }
+    return success;
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
