@@ -126,6 +126,7 @@ void VulkanDecoderBase::DispatchResizeWindowCommand2(
 
 void VulkanDecoderBase::DispatchCreateHardwareBufferCommand(
     format::ThreadId                                    thread_id,
+    format::HandleId                                    device_id,
     format::HandleId                                    memory_id,
     uint64_t                                            buffer_id,
     uint32_t                                            format,
@@ -141,7 +142,7 @@ void VulkanDecoderBase::DispatchCreateHardwareBufferCommand(
     for (auto consumer : consumers_)
     {
         consumer->ProcessCreateHardwareBufferCommand(
-            memory_id, buffer_id, format, width, height, stride, usage, layers, plane_info);
+            device_id, memory_id, buffer_id, format, width, height, stride, usage, layers, plane_info);
     }
 }
 
@@ -568,6 +569,14 @@ void VulkanDecoderBase::SetCurrentBlockIndex(uint64_t block_index)
     }
 }
 
+void VulkanDecoderBase::SetCurrentFrameNumber(uint64_t frame_number)
+{
+    for (auto consumer : consumers_)
+    {
+        consumer->SetCurrentFrameNumber(frame_number);
+    }
+}
+
 void VulkanDecoderBase::DispatchVulkanAccelerationStructuresBuildMetaCommand(const uint8_t* parameter_buffer,
                                                                              size_t         buffer_size)
 {
