@@ -172,6 +172,13 @@ struct VulkanReplayDeviceInfo
     std::optional<VkPhysicalDeviceDriverProperties>                   driver_properties;
     std::optional<VkPhysicalDeviceRayTracingPipelinePropertiesKHR>    raytracing_properties;
     std::optional<VkPhysicalDeviceAccelerationStructurePropertiesKHR> acceleration_structure_properties;
+
+    bool IsPropertiesNull()
+    {
+        // Not include memory properties.
+        return properties == std::nullopt || driver_properties == std::nullopt ||
+               raytracing_properties == std::nullopt || acceleration_structure_properties == std::nullopt;
+    }
 };
 
 template <typename T>
@@ -228,7 +235,6 @@ typedef VulkanObjectInfo<VkDisplayModeKHR>                     VulkanDisplayMode
 typedef VulkanObjectInfo<VkDebugReportCallbackEXT>             VulkanDebugReportCallbackEXTInfo;
 typedef VulkanObjectInfo<VkIndirectCommandsLayoutNV>           VulkanIndirectCommandsLayoutNVInfo;
 typedef VulkanObjectInfo<VkDebugUtilsMessengerEXT>             VulkanDebugUtilsMessengerEXTInfo;
-typedef VulkanObjectInfo<VkAccelerationStructureNV>            VulkanAccelerationStructureNVInfo;
 typedef VulkanObjectInfo<VkPerformanceConfigurationINTEL>      VulkanPerformanceConfigurationINTELInfo;
 typedef VulkanObjectInfo<VkMicromapEXT>                        VulkanMicromapEXTInfo;
 typedef VulkanObjectInfo<VkOpticalFlowSessionNV>               VulkanOpticalFlowSessionNVInfo;
@@ -303,33 +309,56 @@ struct VulkanPhysicalDeviceInfo : public VulkanObjectInfo<VkPhysicalDevice>
 
     // When Non-null, the GetVkObject will recur on the alias Id
     format::HandleId vulkan_alias{ format::kNullHandleId };
+
+    // keep track of queried surface-formats
+    std::optional<std::vector<VkSurfaceFormatKHR>> surface_formats;
 };
 
 struct VulkanDeviceInfo : public VulkanObjectInfo<VkDevice>
 {
     VkPhysicalDevice                         parent{ VK_NULL_HANDLE };
-    std::unique_ptr<VulkanResourceAllocator> allocator;
+    std::shared_ptr<VulkanResourceAllocator> allocator;
     std::unordered_map<uint32_t, size_t>     array_counts;
 
     std::unordered_map<format::HandleId, uint64_t> opaque_addresses;
 
     // Map pipeline ID to ray tracing shader group handle capture replay data.
-    std::unordered_map<format::HandleId, const std::vector<uint8_t>> shader_group_handles;
+    std::unordered_map<format::HandleId, std::vector<uint8_t>> shader_group_handles;
 
     // The following values are only used when loading the initial state for trimmed files.
     std::vector<std::string>                   extensions;
-    std::unique_ptr<VulkanResourceInitializer> resource_initializer;
+    std::shared_ptr<VulkanResourceInitializer> resource_initializer;
 
     // Physical device property & feature state at device creation
     graphics::VulkanDevicePropertyFeatureInfo property_feature_info;
 
-    std::unordered_map<uint32_t, VkDeviceQueueCreateFlags> queue_family_creation_flags;
-    std::vector<bool>                                      queue_family_index_enabled;
+    struct EnabledQueueFamilyFlags
+    {
+        std::unordered_map<uint32_t, VkDeviceQueueCreateFlags> queue_family_creation_flags;
+        std::unordered_map<uint32_t, VkDeviceQueueCreateFlags> queue_family_properties_flags;
+
+        std::vector<bool> queue_family_index_enabled;
+    } enabled_queue_family_flags;
 
     std::vector<VkPhysicalDevice> replay_device_group;
 
     // For use with device deduplication
-    bool is_duplicate{ false };
+    format::HandleId duplicate_source_id{ format::kNullHandleId };
+
+    void copy_characteristics(const VulkanDeviceInfo* source_info)
+    {
+        parent                     = source_info->parent;
+        allocator                  = source_info->allocator;
+        array_counts               = source_info->array_counts;
+        opaque_addresses           = source_info->opaque_addresses;
+        shader_group_handles       = source_info->shader_group_handles;
+        extensions                 = source_info->extensions;
+        resource_initializer       = source_info->resource_initializer;
+        property_feature_info      = source_info->property_feature_info;
+        enabled_queue_family_flags = source_info->enabled_queue_family_flags;
+        replay_device_group        = source_info->replay_device_group;
+        duplicate_source_id        = source_info->capture_id;
+    }
 };
 
 struct VulkanQueueInfo : public VulkanObjectInfo<VkQueue>
@@ -368,6 +397,7 @@ struct VulkanDeviceMemoryInfo : public VulkanObjectInfo<VkDeviceMemory>
     VulkanResourceAllocator::MemoryData allocator_data{ 0 };
 };
 
+struct VulkanAccelerationStructureKHRInfo;
 struct VulkanBufferInfo : public VulkanObjectInfo<VkBuffer>
 {
     // The following values are only used for memory portability.
@@ -378,9 +408,15 @@ struct VulkanBufferInfo : public VulkanObjectInfo<VkBuffer>
     // This is only used when loading the initial state for trimmed files.
     VkMemoryPropertyFlags memory_property_flags{ 0 };
 
+    std::vector<VkMemoryPropertyFlags> sparse_memory_property_flags;
+
     VkBufferUsageFlags usage{ 0 };
     VkDeviceSize       size{ 0 };
     uint32_t           queue_family_index{ 0 };
+
+    // map acceleration-structure capture-addresses to existing (alias) AS-handles
+    std::unordered_map<VkDeviceAddress, std::unordered_set<const VulkanAccelerationStructureKHRInfo*>>
+        acceleration_structures;
 };
 
 struct VulkanBufferViewInfo : public VulkanObjectInfo<VkBufferView>
@@ -405,6 +441,8 @@ struct VulkanImageInfo : public VulkanObjectInfo<VkImage>
     // This is only used when loading the initial state for trimmed files.
     VkMemoryPropertyFlags memory_property_flags{ 0 };
 
+    std::vector<VkMemoryPropertyFlags> sparse_memory_property_flags;
+
     VkImageUsageFlags     usage{ 0 };
     VkImageType           type{};
     VkFormat              format{};
@@ -616,7 +654,7 @@ struct VulkanVideoSessionKHRInfo : VulkanObjectInfo<VkVideoSessionKHR>
     std::unordered_map<uint32_t, size_t> array_counts;
 
     // The following values are only used for memory portability.
-    std::vector<VulkanResourceAllocator::ResourceData> allocator_datas;
+    VulkanResourceAllocator::ResourceData allocator_data;
 
     // This is only used when loading the initial state for trimmed files.
     std::vector<VkMemoryPropertyFlags> memory_property_flags;
@@ -705,7 +743,18 @@ struct VulkanAccelerationStructureKHRInfo : public VulkanObjectInfo<VkAccelerati
     VkAccelerationStructureTypeKHR type = VK_ACCELERATION_STRUCTURE_TYPE_MAX_ENUM_KHR;
 
     //! associated buffer
-    VkBuffer buffer = VK_NULL_HANDLE;
+    VkBuffer     buffer = VK_NULL_HANDLE;
+    VkDeviceSize offset = 0;
+    VkDeviceSize size   = 0;
+};
+
+struct VulkanAccelerationStructureNVInfo : public VulkanObjectInfo<VkAccelerationStructureNV>
+{
+    // The following values are only used for memory portability.
+    VulkanResourceAllocator::ResourceData allocator_data{ 0 };
+
+    // This is only used when loading the initial state for trimmed files.
+    VkMemoryPropertyFlags memory_property_flags{ 0 };
 };
 
 //
