@@ -34,11 +34,13 @@
 #include "generated/generated_vulkan_dispatch_table.h"
 #include "format/format.h"
 #include "generated/generated_vulkan_struct_decoders.h"
+#include "util/compressor.h"
 #include "util/defines.h"
 #include "vulkan/vulkan_core.h"
 
 #include <cstdint>
 #include <type_traits>
+#include <memory>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
@@ -315,6 +317,12 @@ class VulkanReplayDumpResourcesBase
                                        uint32_t                        dynamicOffsetCount,
                                        const uint32_t*                 pDynamicOffsets);
 
+    void
+    OverrideCmdBindDescriptorSets2(const ApiCallInfo&                                      call_info,
+                                   PFN_vkCmdBindDescriptorSets2                            func,
+                                   VkCommandBuffer                                         original_command_buffer,
+                                   StructPointerDecoder<Decoded_VkBindDescriptorSetsInfo>* pBindDescriptorSetsInfo);
+
     void OverrideCmdBindIndexBuffer(const ApiCallInfo&       call_info,
                                     PFN_vkCmdBindIndexBuffer func,
                                     VkCommandBuffer          original_command_buffer,
@@ -562,6 +570,8 @@ class VulkanReplayDumpResourcesBase
     VulkanDumpResourcesDelegate*                        user_delegate_;
     VulkanDumpResourcesDelegate*                        active_delegate_;
 
+    std::unique_ptr<util::Compressor> compressor_;
+
     std::string capture_filename;
 
     std::function<void(const char*)> fatal_error_handler_;
