@@ -28,7 +28,7 @@
 #include "decode/vulkan_object_info.h"
 #include "decode/vulkan_replay_options.h"
 #include "generated/generated_vulkan_dispatch_table.h"
-#include "format/format.h"
+#include "util/compressor.h"
 #include "util/defines.h"
 #include "util/logging.h"
 #include "vulkan/vulkan_core.h"
@@ -36,7 +36,6 @@
 #include <cstdint>
 #include <memory>
 #include <unordered_map>
-#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -51,7 +50,8 @@ class DispatchTraceRaysDumpingContext
                                     const std::vector<uint64_t>* trace_rays_indices,
                                     CommonObjectInfoTable&       object_info_table,
                                     const VulkanReplayOptions&   options,
-                                    VulkanDumpResourcesDelegate& delegate);
+                                    VulkanDumpResourcesDelegate& delegate,
+                                    const util::Compressor*      compressor);
 
     ~DispatchTraceRaysDumpingContext();
 
@@ -74,8 +74,12 @@ class DispatchTraceRaysDumpingContext
                             uint32_t                                           dynamicOffsetCount,
                             const uint32_t*                                    pDynamicOffsets);
 
-    VkResult DumpDispatchTraceRays(
-        VkQueue queue, uint64_t qs_index, uint64_t bcb_index, const VkSubmitInfo& submit_info, VkFence fence);
+    VkResult DumpDispatchTraceRays(VkQueue             queue,
+                                   uint64_t            qs_index,
+                                   uint64_t            bcb_index,
+                                   const VkSubmitInfo& submit_info,
+                                   VkFence             fence,
+                                   bool                use_semaphores);
 
     VkResult DumpMutableResources(uint64_t bcb_index, uint64_t qs_index, uint64_t cmd_index, bool is_dispatch);
 
@@ -139,9 +143,9 @@ class DispatchTraceRaysDumpingContext
     VkCommandBuffer                DR_command_buffer_;
     std::vector<uint64_t>          dispatch_indices_;
     std::vector<uint64_t>          trace_rays_indices_;
-    bool                           dump_resources_before_;
     VulkanDumpResourcesDelegate&   delegate_;
-    bool                           dump_immutable_resources_;
+    const VulkanReplayOptions&     options_;
+    const util::Compressor*        compressor_;
 
     // One entry per descriptor set for each compute and ray tracing binding points
     BoundDescriptorSets bound_descriptor_sets_compute_;
@@ -159,29 +163,27 @@ class DispatchTraceRaysDumpingContext
 
         struct ImageContext
         {
-            const VulkanImageInfo* original_image{ nullptr };
-            VkImage                image{ VK_NULL_HANDLE };
-            VkDeviceMemory         image_memory{ VK_NULL_HANDLE };
-            VkShaderStageFlags     stages;
-            VkDescriptorType       desc_type;
-            uint32_t               desc_set;
-            uint32_t               desc_binding;
-            uint32_t               array_index;
+            VulkanImageInfo    new_image_info;
+            VkDeviceMemory     image_memory{ VK_NULL_HANDLE };
+            VkShaderStageFlags stages;
+            VkDescriptorType   desc_type;
+            uint32_t           desc_set;
+            uint32_t           desc_binding;
+            uint32_t           array_index;
         };
 
         std::vector<ImageContext> images;
 
         struct BufferContext
         {
-            const VulkanBufferInfo* original_buffer{ nullptr };
-            VkBuffer                buffer{ VK_NULL_HANDLE };
-            VkDeviceMemory          buffer_memory{ VK_NULL_HANDLE };
-            VkDeviceSize            cloned_size{ 0 };
-            VkShaderStageFlags      stages;
-            VkDescriptorType        desc_type;
-            uint32_t                desc_set;
-            uint32_t                desc_binding;
-            uint32_t                array_index;
+            VulkanBufferInfo   new_buffer_info;
+            VkDeviceMemory     buffer_memory{ VK_NULL_HANDLE };
+            VkDeviceSize       cloned_size{ 0 };
+            VkShaderStageFlags stages;
+            VkDescriptorType   desc_type;
+            uint32_t           desc_set;
+            uint32_t           desc_binding;
+            uint32_t           array_index;
         };
 
         std::vector<BufferContext> buffers;
@@ -261,11 +263,7 @@ class DispatchTraceRaysDumpingContext
                 VkBuffer       new_params_buffer;
                 VkDeviceMemory new_params_memory;
 
-                // Pointers that will point to host allocated memory and filled with the dispatch
-                // params read back after executing on the gpu. Because of the union a data
-                // structure with a non default destructor (vector/unique_ptr) cannot be used
-                // and we will handle the memory managment ourselves.
-                DispatchParams* dispatch_params;
+                DispatchParams fetched_dispatch_params;
             };
 
             DispatchIndirect dispatch_indirect;
@@ -275,7 +273,7 @@ class DispatchTraceRaysDumpingContext
             {}
 
             DispatchParamsUnion(const VulkanBufferInfo* params_buffer_info, VkDeviceSize offset) :
-                dispatch_indirect{ params_buffer_info, offset, VK_NULL_HANDLE, VK_NULL_HANDLE, nullptr }
+                dispatch_indirect{ params_buffer_info, offset, VK_NULL_HANDLE, VK_NULL_HANDLE }
             {}
 
             DispatchParamsUnion(uint32_t baseGroupX,
@@ -324,6 +322,8 @@ class DispatchTraceRaysDumpingContext
         // Need to keep track if a dispatch context from a secondary command buffer has been updated with information
         // that might be available only from the primary command buffer
         bool updated_referenced_descriptors;
+
+        DumpedResourcesInfo dumped_resources;
     };
 
     enum TraceRaysTypes
@@ -436,6 +436,8 @@ class DispatchTraceRaysDumpingContext
         // Need to keep track if a trace rays context from a secondary command buffer has been updated with information
         // that might be available only from the primary command buffer
         bool updated_referenced_descriptors;
+
+        DumpedResourcesInfo dumped_resources;
     };
 
   private:
@@ -455,9 +457,8 @@ class DispatchTraceRaysDumpingContext
     // multiple times
     struct DumpedDescriptors
     {
-        std::unordered_set<const VulkanImageInfo*>      image_descriptors;
-        std::unordered_set<const VulkanBufferInfo*>     buffer_descriptors;
-        std::unordered_set<const std::vector<uint8_t>*> inline_uniform_blocks;
+        std::unordered_map<const VulkanImageInfo*, const DumpedImage&>   image_descriptors;
+        std::unordered_map<const VulkanBufferInfo*, const DumpedBuffer&> buffer_descriptors;
     };
 
     DumpedDescriptors dispatch_dumped_descriptors_;
