@@ -28,7 +28,7 @@
 #include "decode/vulkan_object_info.h"
 #include "decode/vulkan_replay_options.h"
 #include "generated/generated_vulkan_dispatch_table.h"
-#include "format/format.h"
+#include "util/compressor.h"
 #include "util/defines.h"
 #include "vulkan/vulkan_core.h"
 
@@ -44,6 +44,9 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 class DrawCallsDumpingContext
 {
   public:
+    // Forward declaration
+    struct DrawCallParams;
+
     enum DrawCallType
     {
         kDraw,
@@ -62,7 +65,8 @@ class DrawCallsDumpingContext
                             const RenderPassIndices*     rp_indices,
                             CommonObjectInfoTable&       object_info_table,
                             const VulkanReplayOptions&   options,
-                            VulkanDumpResourcesDelegate& delegate);
+                            VulkanDumpResourcesDelegate& delegate,
+                            const util::Compressor*      compressor);
 
     ~DrawCallsDumpingContext();
 
@@ -132,7 +136,8 @@ class DrawCallsDumpingContext
                          VkIndexType             index_type,
                          VkDeviceSize            size = 0);
 
-    void FinalizeCommandBuffer();
+    // When this is called for a command buffer that corresponds to a before command, dc_params should be null
+    void FinalizeCommandBuffer(DrawCallParams* dc_params = nullptr);
 
     uint32_t GetDrawCallActiveCommandBuffers(CommandBufferIterator& first, CommandBufferIterator& last) const;
 
@@ -142,36 +147,27 @@ class DrawCallsDumpingContext
     VkResult DumpRenderTargetAttachments(
         uint64_t cmd_buf_index, uint64_t rp, uint64_t sp, uint64_t qs_index, uint64_t bcb_index);
 
-    VkResult DumpImmutableDescriptors(uint64_t qs_index, uint64_t bcb_index, uint64_t dc_index, uint64_t rp);
+    VkResult DumpDescriptors(uint64_t qs_index, uint64_t bcb_index, uint64_t dc_index, uint64_t rp);
 
     VkResult DumpVertexIndexBuffers(uint64_t qs_index, uint64_t bcb_index, uint64_t dc_index);
 
-    void InsertNewDrawParameters(
+    DrawCallParams* InsertNewDrawParameters(
         uint64_t index, uint32_t vertex_count, uint32_t instance_count, uint32_t first_vertex, uint32_t first_instance);
 
-    void InsertNewDrawIndexedParameters(uint64_t index,
-                                        uint32_t index_count,
-                                        uint32_t instance_count,
-                                        uint32_t first_index,
-                                        int32_t  vertexOffset,
-                                        uint32_t first_instance);
+    DrawCallParams* InsertNewDrawIndexedParameters(uint64_t index,
+                                                   uint32_t index_count,
+                                                   uint32_t instance_count,
+                                                   uint32_t first_index,
+                                                   int32_t  vertexOffset,
+                                                   uint32_t first_instance);
 
-    void InsertNewDrawIndirectParameters(
+    DrawCallParams* InsertNewDrawIndirectParameters(
         uint64_t index, const VulkanBufferInfo* buffer_info, VkDeviceSize offset, uint32_t draw_count, uint32_t stride);
 
-    void InsertNewDrawIndexedIndirectParameters(
+    DrawCallParams* InsertNewDrawIndexedIndirectParameters(
         uint64_t index, const VulkanBufferInfo* buffer_info, VkDeviceSize offset, uint32_t draw_count, uint32_t stride);
 
-    void InsertNewIndirectCountParameters(uint64_t                index,
-                                          const VulkanBufferInfo* buffer_info,
-                                          VkDeviceSize            offset,
-                                          const VulkanBufferInfo* count_buffer_info,
-                                          VkDeviceSize            count_buffer_offset,
-                                          uint32_t                max_draw_count,
-                                          uint32_t                stride,
-                                          DrawCallType            drawcall_type);
-
-    void InsertNewDrawIndexedIndirectCountParameters(uint64_t                index,
+    DrawCallParams* InsertNewIndirectCountParameters(uint64_t                index,
                                                      const VulkanBufferInfo* buffer_info,
                                                      VkDeviceSize            offset,
                                                      const VulkanBufferInfo* count_buffer_info,
@@ -180,6 +176,15 @@ class DrawCallsDumpingContext
                                                      uint32_t                stride,
                                                      DrawCallType            drawcall_type);
 
+    DrawCallParams* InsertNewDrawIndexedIndirectCountParameters(uint64_t                index,
+                                                                const VulkanBufferInfo* buffer_info,
+                                                                VkDeviceSize            offset,
+                                                                const VulkanBufferInfo* count_buffer_info,
+                                                                VkDeviceSize            count_buffer_offset,
+                                                                uint32_t                max_draw_count,
+                                                                uint32_t                stride,
+                                                                DrawCallType            drawcall_type);
+
     void Release();
 
     const std::vector<VkCommandBuffer>& GetCommandBuffers() const { return command_buffers_; }
@@ -225,13 +230,9 @@ class DrawCallsDumpingContext
     const VulkanPipelineInfo*    bound_gr_pipeline_;
     uint32_t                     current_renderpass_;
     uint32_t                     current_subpass_;
-    bool                         dump_resources_before_;
     VulkanDumpResourcesDelegate& delegate_;
-    bool                         dump_depth_;
-    int32_t                      color_attachment_to_dump_;
-    bool                         dump_vertex_index_buffers_;
-    bool                         dump_immutable_resources_;
-    bool                         dump_unused_vertex_bindings_;
+    const VulkanReplayOptions&   options_;
+    const util::Compressor*      compressor_;
 
     // Execute commands block index : DrawCallContexts
     std::unordered_map<uint64_t, std::vector<DrawCallsDumpingContext*>> secondaries_;
@@ -310,14 +311,14 @@ class DrawCallsDumpingContext
     {
         struct BufferPerBinding
         {
-            BufferPerBinding() : buffer_info(nullptr), offset(0), size(0), stride(0), actual_size(0) {}
+            BufferPerBinding() : buffer_info(nullptr), offset(0), size(0), stride(0) {}
 
             BufferPerBinding(const VulkanBufferInfo* buffer_info,
                              VkDeviceSize            offset,
                              VkDeviceSize            size   = 0,
                              VkDeviceSize            stride = 0) :
                 buffer_info(buffer_info),
-                offset(offset), size(size), stride(stride), actual_size(0)
+                offset(offset), size(size), stride(stride)
             {}
 
             const VulkanBufferInfo* buffer_info;
@@ -326,10 +327,6 @@ class DrawCallsDumpingContext
             // These are provided only by CmdBindVertexBuffers2
             VkDeviceSize size;
             VkDeviceSize stride;
-
-            // This is the size actually used as an vertex buffer from all referencing draw calls
-            // and is calculated based on the indices (if an index buffer is used)
-            VkDeviceSize actual_size;
         };
 
         // One entry for each vertex buffer bound at each binding
@@ -343,16 +340,14 @@ class DrawCallsDumpingContext
     // Keep track of bound index buffer
     struct BoundIndexBuffer
     {
-        BoundIndexBuffer() :
-            buffer_info(nullptr), offset(0), index_type(VK_INDEX_TYPE_MAX_ENUM), size(0), actual_size(0)
-        {}
+        BoundIndexBuffer() : buffer_info(nullptr), offset(0), index_type(VK_INDEX_TYPE_MAX_ENUM), size(0) {}
 
         BoundIndexBuffer(const VulkanBufferInfo* buffer_info,
                          VkDeviceSize            offset,
                          VkIndexType             index_type,
                          VkDeviceSize            size) :
             buffer_info(buffer_info),
-            offset(offset), index_type(index_type), size(size), actual_size(0)
+            offset(offset), index_type(index_type), size(size)
         {}
 
         const VulkanBufferInfo* buffer_info;
@@ -361,9 +356,6 @@ class DrawCallsDumpingContext
 
         // This is provided only by vkCmdBindIndexBuffer2KHR
         VkDeviceSize size;
-
-        // This is the size actually used as an index buffer from all referencing draw calls
-        VkDeviceSize actual_size;
     };
 
   private:
@@ -657,28 +649,13 @@ class DrawCallsDumpingContext
         // Keep copies of the descriptor bindings referenced by each draw call
         BoundDescriptorSets referenced_descriptors;
 
-        // These are used to store information calculated when dumping vertex and index buffers.
-        // This information is latter used when writting the output json file.
-        struct
-        {
-            struct
-            {
-                bool   dumped{ false };
-                size_t offset{ 0 };
-            } index_buffer_info;
-
-            struct VertexBufferBindingInfo
-            {
-                size_t offset{ 0 };
-            };
-            std::unordered_map<uint32_t, VertexBufferBindingInfo> vertex_bindings_info;
-        } json_output_info;
-
-        // Need to keep track if a draw call context from a secondary command buffer has been updated with information
-        // that might be available only from the primary command buffer
+        // Need to keep track if a draw call context from a secondary command buffer has been updated with
+        // information that might be available only from the primary command buffer
         bool updated_bound_vertex_buffers;
         bool updated_bound_index_buffer;
         bool updated_referenced_descriptors;
+
+        DumpedResourcesInfo dumped_resources;
     };
 
   private:
@@ -706,9 +683,8 @@ class DrawCallsDumpingContext
     // multiple times
     struct RenderPassDumpedDescriptors
     {
-        std::unordered_set<const VulkanImageInfo*>      image_descriptors;
-        std::unordered_set<const VulkanBufferInfo*>     buffer_descriptors;
-        std::unordered_set<const std::vector<uint8_t>*> inline_uniform_blocks;
+        std::unordered_map<const VulkanImageInfo*, const DumpedImage&>   image_descriptors;
+        std::unordered_map<const VulkanBufferInfo*, const DumpedBuffer&> buffer_descriptors;
     };
 
     std::vector<RenderPassDumpedDescriptors> render_pass_dumped_descriptors_;
