@@ -27,6 +27,7 @@
 #include "decode/replay_options.h"
 
 #include "decode/vulkan_resource_allocator.h"
+#include "format/format.h"
 #include "util/defines.h"
 
 #include <cstdint>
@@ -67,10 +68,12 @@ static constexpr int kUnspecifiedColorAttachment = -1;
 struct VulkanReplayOptions : public ReplayOptions
 {
     bool                         enable_vulkan{ true };
+    bool                         capture{ false };
     bool                         omit_pipeline_cache_data{ false };
     bool                         use_colorspace_fallback{ false };
     bool                         offscreen_swapchain_frame_boundary{ false };
     util::SwapchainOption        swapchain_option{ util::SwapchainOption::kVirtual };
+    util::PresentModeOption      present_mode_option{ util::PresentModeOption::kCapture };
     bool                         virtual_swapchain_skip_blit{ false };
     int32_t                      override_gpu_group_index{ -1 };
     int32_t                      surface_index{ -1 };
@@ -111,6 +114,8 @@ struct VulkanReplayOptions : public ReplayOptions
     bool  dump_resources_dump_separate_alpha{ false };
     bool  dump_resources_dump_unused_vertex_bindings{ false };
 
+    format::CompressionType dump_resources_binary_file_compression_type{ format::CompressionType::kNone };
+
     bool preload_measurement_range{ false };
 
     std::string load_pipeline_cache_filename;
