@@ -372,16 +372,6 @@ struct CustomWrapperPreCall<format::ApiCallId::ApiCall_ID3D12Resource_Unmap>
     }
 };
 
-template <>
-struct CustomWrapperPostCall<format::ApiCallId::ApiCall_ID3D12Resource_GetHeapProperties>
-{
-    template <typename... Args>
-    static void Dispatch(D3D12CaptureManager* manager, Args... args)
-    {
-        manager->PostProcess_ID3D12Resource_GetHeapProperties(args...);
-    }
-};
-
 template <>
 struct CustomWrapperPostCall<format::ApiCallId::ApiCall_ID3D12Resource_GetGPUVirtualAddress>
 {
@@ -398,16 +388,6 @@ inline void CustomWrapperDestroyCall<ID3D12Resource_Wrapper>(ID3D12Resource_Wrap
     D3D12CaptureManager::Get()->Destroy_ID3D12Resource(wrapper);
 }
 
-template <>
-struct CustomWrapperPostCall<format::ApiCallId::ApiCall_ID3D12Heap_GetDesc>
-{
-    template <typename... Args>
-    static void Dispatch(D3D12CaptureManager* manager, Args... args)
-    {
-        manager->PostProcess_ID3D12Heap_GetDesc(args...);
-    }
-};
-
 template <>
 struct CustomWrapperPreCall<format::ApiCallId::ApiCall_ID3D12CommandQueue_ExecuteCommandLists>
 {
@@ -834,6 +814,16 @@ struct CustomWrapperPostCall<format::ApiCallId::ApiCall_ID3D12Object_SetName>
     }
 };
 
+template <>
+struct CustomWrapperPostCall<format::ApiCallId::ApiCall_ID3D12GraphicsCommandList4_InitializeMetaCommand>
+{
+    template <typename... Args>
+    static void Dispatch(D3D12CaptureManager* manager, Args... args)
+    {
+        manager->PostProcess_InitializeMetaCommand(args...);
+    }
+};
+
 GFXRECON_END_NAMESPACE(encode)
 GFXRECON_END_NAMESPACE(gfxrecon)
 
