@@ -425,6 +425,7 @@ struct ID3D12ResourceInfo : public DxWrapperInfo
 
     ID3D12Heap_Wrapper* heap_wrapper{ nullptr };
     uint64_t            heap_offset;
+    uint64_t            heap_id{ format::kNullHandleId };
 
     IDXGISwapChain_Wrapper* swapchain_wrapper{ nullptr };
 };
@@ -437,12 +438,16 @@ struct ID3D12HeapInfo : public DxWrapperInfo
     D3D12_MEMORY_POOL         memory_pool{};
     uint64_t                  heap_size{ 0 };
     D3D12_GPU_VIRTUAL_ADDRESS gpu_va{ 0 };
+    D3D12_HEAP_FLAGS          heap_flags{ D3D12_HEAP_FLAG_NONE };
 
     const void* open_existing_address{ nullptr }; ///< Address used to create heap with OpenExistingHeapFromAddress.
 };
 
 struct ID3D12MetaCommandInfo : public DxWrapperInfo
-{};
+{
+    bool                                      was_initialized{ false };
+    std::unique_ptr<util::MemoryOutputStream> initialize_parameters;
+};
 
 struct ID3D12ShaderCacheSessionInfo : public DxWrapperInfo
 {};
@@ -498,7 +503,8 @@ struct ID3D12CommandListInfo : public DxWrapperInfo
     std::array<graphics::dx12::CommandSet, 3>    split_command_sets;
     bool                                         is_split_commandlist{ false };
     uint32_t                                     find_target_draw_call_count{ 0 };
-    std::shared_ptr<const ID3D12CommandListInfo> target_bundle_commandlist_info{ false };
+    std::shared_ptr<const ID3D12CommandListInfo> target_bundle_commandlist_info;
+    bool                                         is_trim_target{ false };
 };
 
 struct ID3D10BlobInfo : public DxWrapperInfo
@@ -552,6 +558,18 @@ struct ID3D12DSRDeviceFactoryInfo : public DxWrapperInfo
 struct ID3D12ManualWriteTrackingResourceInfo : public DxWrapperInfo
 {};
 
+struct ID3D12WorkGraphPropertiesInfo : public DxWrapperInfo
+{};
+
+struct ID3D12PageableToolsInfo : public DxWrapperInfo
+{};
+
+struct ID3D12DeviceToolsInfo : public DxWrapperInfo
+{};
+
+struct ID3D12GBVDiagnosticsInfo : public DxWrapperInfo
+{};
+
 struct AgsContextInfo : public DxWrapperInfo
 {};
 
