@@ -118,7 +118,7 @@ void Dx12StateWriter::WriteState(const Dx12StateTable& state_table, uint64_t fra
     WriteFenceState(state_table);
 
     // Heaps
-    StandardCreateWrite<ID3D10Blob_Wrapper>(state_table);
+    WriteRootSignatureBlobState(state_table);
     WriteHeapState(state_table);
 
     // Root signatures
@@ -153,6 +153,7 @@ void Dx12StateWriter::WriteState(const Dx12StateTable& state_table, uint64_t fra
     // Pipelines
     StandardCreateWrite<ID3D12PipelineLibrary_Wrapper>(state_table);
     StandardCreateWrite<ID3D12PipelineState_Wrapper>(state_table);
+    WriteCachedPSOBlobState(state_table);
 
     // Debug objects
     StandardCreateWrite<ID3D12Debug2_Wrapper>(state_table);
@@ -361,6 +362,49 @@ void Dx12StateWriter::WriteMethodCall(format::ApiCallId         call_id,
     output_stream_->Write(data_pointer, data_size);
 }
 
+bool Dx12StateWriter::IsCachedPSOBlob(const ID3D10Blob_Wrapper* wrapper) const
+{
+    GFXRECON_ASSERT(wrapper != nullptr);
+
+    auto wrapper_info = wrapper->GetObjectInfo();
+    GFXRECON_ASSERT(wrapper_info != nullptr);
+
+    return (wrapper_info->create_call_id == format::ApiCall_ID3D12PipelineState_GetCachedBlob);
+}
+
+void Dx12StateWriter::WriteRootSignatureBlobState(const Dx12StateTable& state_table)
+{
+    std::set<util::MemoryOutputStream*> processed;
+    state_table.VisitWrappers([&](const ID3D10Blob_Wrapper* wrapper) {
+        GFXRECON_ASSERT(wrapper != nullptr);
+
+        if (IsRootSignatureBlob(wrapper))
+        {
+            // Filter duplicate entries for calls that create multiple objects, where objects created by the same call
+            // all reference the same parameter buffer.
+            auto wrapper_info = wrapper->GetObjectInfo();
+            GFXRECON_ASSERT((wrapper_info != nullptr) && (wrapper_info->create_parameters != nullptr));
+
+            if (processed.find(wrapper_info->create_parameters.get()) == processed.end())
+            {
+                StandardCreateWrite(wrapper);
+                processed.insert(wrapper_info->create_parameters.get());
+            }
+        }
+    });
+}
+
+void Dx12StateWriter::WriteCachedPSOBlobState(const Dx12StateTable& state_table)
+{
+    std::set<util::MemoryOutputStream*> processed;
+    state_table.VisitWrappers([&](const ID3D10Blob_Wrapper* wrapper) {
+        if (IsCachedPSOBlob(wrapper))
+        {
+            StandardCreateWrite(wrapper);
+        }
+    });
+}
+
 void Dx12StateWriter::WriteHeapState(const Dx12StateTable& state_table)
 {
     std::set<util::MemoryOutputStream*> processed;
@@ -381,9 +425,33 @@ void Dx12StateWriter::WriteHeapState(const Dx12StateTable& state_table)
         }
 
         StandardCreateWrite(wrapper);
+        if (wrapper_info->heap_flags & D3D12_HEAP_FLAG_CREATE_NOT_RESIDENT)
+        {
+            WriteHeapMakeResidentCmd(wrapper);
+        }
     });
 }
 
+void Dx12StateWriter::WriteHeapMakeResidentCmd(const ID3D12Heap_Wrapper* wrapper)
+{
+    GFXRECON_ASSERT(wrapper != nullptr);
+    GFXRECON_ASSERT(wrapper->GetObjectInfo() != nullptr);
+
+    auto wrapper_info = wrapper->GetObjectInfo();
+
+    UINT                  num_objects  = 1;
+    HRESULT               return_value = S_OK;
+    const ID3D12Pageable* ppObjects[1];
+    ppObjects[0] = reinterpret_cast<const ID3D12Pageable*>(wrapper);
+
+    encoder_.EncodeUInt32Value(num_objects);
+    encoder_.EncodeObjectArray(ppObjects, num_objects);
+    encoder_.EncodeInt32Value(return_value);
+    WriteMethodCall(
+        format::ApiCallId::ApiCall_ID3D12Device_MakeResident, wrapper_info->create_object_id, &parameter_stream_);
+    parameter_stream_.Clear();
+}
+
 bool Dx12StateWriter::WriteCreateHeapAllocationCmd(const void* address)
 {
     MEMORY_BASIC_INFORMATION info{};
@@ -1067,6 +1135,9 @@ void Dx12StateWriter::WriteCommandListState(const Dx12StateTable& state_table)
     std::vector<ID3D12CommandList_Wrapper*> direct_command_lists;
     std::vector<ID3D12CommandList_Wrapper*> open_command_lists;
 
+    const bool trim_to_draw_enabled =
+        (D3D12CaptureManager::Get()->GetTrimBoundary() == CaptureSettings::TrimBoundary::kDrawCalls);
+
     state_table.VisitWrappers([&](ID3D12CommandList_Wrapper* list_wrapper) {
         GFXRECON_ASSERT(list_wrapper != nullptr);
         GFXRECON_ASSERT(list_wrapper->GetWrappedObject() != nullptr);
@@ -1078,6 +1149,10 @@ void Dx12StateWriter::WriteCommandListState(const Dx12StateTable& state_table)
         GFXRECON_ASSERT(list_info->create_parameters != nullptr);
         GFXRECON_ASSERT(list_info->create_object_id != format::kNullHandleId);
 
+        // When trim to draw is enabled, skip command lists that do not contain the target draw calls.
+        if (trim_to_draw_enabled && !list_info->is_trim_target)
+            return;
+
         // Write create calls and commands for bundle command lists. Keep track of primary and open command lists to be
         // written afterward.
         if (list->GetType() == D3D12_COMMAND_LIST_TYPE_BUNDLE)
