@@ -1,7 +1,7 @@
 /*
 ** Copyright (c) 2018-2022 Valve Corporation
 ** Copyright (c) 2018-2022 LunarG, Inc.
-** Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
+** Copyright (c) 2023-2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -54,6 +54,7 @@ typedef uint64_t AddressEncodeType;
 typedef uint8_t  CharEncodeType;  // Encoding type for UTF-8 strings.
 typedef uint16_t WCharEncodeType; // Encoding type for LPCWSTR (UTF-16) strings.
 typedef uint32_t FormatEncodeType;
+typedef uint32_t D3D_FEATURE_LEVELEncodeType;
 
 typedef HandleEncodeType HandleId;
 typedef uint64_t         ThreadId;
@@ -122,42 +123,46 @@ enum AdapterType
 
 enum class MetaDataType : uint16_t
 {
-    kUnknownMetaDataType                    = 0,
-    kDisplayMessageCommand                  = 1,
-    kFillMemoryCommand                      = 2,
-    kResizeWindowCommand                    = 3,
-    kSetSwapchainImageStateCommand          = 4,
-    kBeginResourceInitCommand               = 5,
-    kEndResourceInitCommand                 = 6,
-    kInitBufferCommand                      = 7,
-    kInitImageCommand                       = 8,
-    kCreateHardwareBufferCommand_deprecated = 9,
-    kDestroyHardwareBufferCommand           = 10,
-    kSetDevicePropertiesCommand             = 11,
-    kSetDeviceMemoryPropertiesCommand       = 12,
-    kResizeWindowCommand2                   = 13,
-    kSetOpaqueAddressCommand                = 14,
-    kSetRayTracingShaderGroupHandlesCommand = 15,
-    kCreateHeapAllocationCommand            = 16,
-    kInitSubresourceCommand                 = 17,
-    kExeFileInfoCommand                     = 18,
-    kInitDx12AccelerationStructureCommand   = 19,
-    kFillMemoryResourceValueCommand         = 20,
-    kDxgiAdapterInfoCommand                 = 21,
-    kDriverInfoCommand                      = 22,
-    kReserved23                             = 23,
-    kCreateHardwareBufferCommand            = 24,
-    kReserved25                             = 25,
-    kDx12RuntimeInfoCommand                 = 26,
-    kParentToChildDependency                = 27,
+    kUnknownMetaDataType                                = 0,
+    kDisplayMessageCommand                              = 1,
+    kFillMemoryCommand                                  = 2,
+    kResizeWindowCommand                                = 3,
+    kSetSwapchainImageStateCommand                      = 4,
+    kBeginResourceInitCommand                           = 5,
+    kEndResourceInitCommand                             = 6,
+    kInitBufferCommand                                  = 7,
+    kInitImageCommand                                   = 8,
+    kCreateHardwareBufferCommand_deprecated             = 9,
+    kDestroyHardwareBufferCommand                       = 10,
+    kSetDevicePropertiesCommand                         = 11,
+    kSetDeviceMemoryPropertiesCommand                   = 12,
+    kResizeWindowCommand2                               = 13,
+    kSetOpaqueAddressCommand                            = 14,
+    kSetRayTracingShaderGroupHandlesCommand             = 15,
+    kCreateHeapAllocationCommand                        = 16,
+    kInitSubresourceCommand                             = 17,
+    kExeFileInfoCommand                                 = 18,
+    kInitDx12AccelerationStructureCommand               = 19,
+    kFillMemoryResourceValueCommand                     = 20,
+    kDxgiAdapterInfoCommand                             = 21,
+    kDriverInfoCommand                                  = 22,
+    kReserved23                                         = 23,
+    kCreateHardwareBufferCommand_deprecated2            = 24,
+    kReserved25                                         = 25,
+    kDx12RuntimeInfoCommand                             = 26,
+    kParentToChildDependency                            = 27,
     kVulkanBuildAccelerationStructuresCommand           = 28,
     kVulkanCopyAccelerationStructuresCommand            = 29,
     kVulkanWriteAccelerationStructuresPropertiesCommand = 30,
-    kReserved31                             = 31,
-    kSetEnvironmentVariablesCommand         = 32,
-    kViewRelativeLocation                   = 33,
-    kExecuteBlocksFromFile                  = 34,
-    kReserved35                             = 35
+    kReserved31                                         = 31,
+    kSetEnvironmentVariablesCommand                     = 32,
+    kViewRelativeLocation                               = 33,
+    kExecuteBlocksFromFile                              = 34,
+    kCreateHardwareBufferCommand                        = 35,
+    kInitializeMetaCommand                              = 36,
+
+    //! reserve values with highest-bit for special purposes
+    kBeginExperimentalReservedRange = 1U << 15U
 };
 
 // MetaDataId is stored in the capture file and its type must be uint32_t to avoid breaking capture file compatibility.
@@ -406,10 +411,27 @@ struct CreateHardwareBufferCommandHeader_deprecated
                            // HardwareBufferLayerInfo records.  When unavailable, 'planes' is zero.
 };
 
+struct CreateHardwareBufferCommandHeader_deprecated2
+{
+    MetaDataHeader meta_header;
+    ThreadId       thread_id;
+    HandleId       memory_id; // Globally unique ID assigned to the buffer for tracking memory modifications.
+    uint64_t       buffer_id; // Address of the buffer object.
+    uint32_t       format;
+    uint32_t       width;
+    uint32_t       height;
+    uint32_t       stride; // Size of a row in pixels.
+    uint64_t       usage;
+    uint32_t       layers;
+    uint32_t       planes; // When additional multi-plane data is available, header is followed by 'planes' count
+                           // HardwareBufferLayerInfo records.  When unavailable, 'planes' is zero.
+};
+
 struct CreateHardwareBufferCommandHeader
 {
     MetaDataHeader meta_header;
     ThreadId       thread_id;
+    HandleId       device_id;
     HandleId       memory_id; // Globally unique ID assigned to the buffer for tracking memory modifications.
     uint64_t       buffer_id; // Address of the buffer object.
     uint32_t       format;
@@ -696,6 +718,48 @@ struct ExecuteBlocksFromFile
     uint32_t filename_length;
 };
 
+struct ViewRelativeLocation
+{
+    format::HandleId session_id;
+    format::HandleId space_id;
+
+    // Locate status
+    uint64_t flags;
+
+    // Orientation
+    float qx;
+    float qy;
+    float qz;
+    float qw;
+
+    // Position
+    float x;
+    float y;
+    float z;
+};
+
+struct ViewRelativeLocationCmd
+{
+    MetaDataHeader   meta_header;
+    format::ThreadId thread_id;
+
+    ViewRelativeLocation location;
+};
+
+struct InitializeMetaCommand
+{
+    MetaDataHeader   meta_header{};
+    ThreadId         thread_id;
+    format::HandleId capture_id;
+    uint32_t         block_index{ 0 };
+    uint32_t         total_number_of_initializemetacommand{ 0 };
+    uint64_t         initialization_parameters_data_size{ 0 };
+
+    // In the capture file, initialize metacommand data is written in the following order:
+    // InitializeMetaCommandHeder
+    // parameters data
+};
+
 // Restore size_t to normal behavior.
 #undef size_t
 
