@@ -25,16 +25,24 @@
 #ifndef GFXRECON_GRAPHICS_DX12_UTIL_H
 #define GFXRECON_GRAPHICS_DX12_UTIL_H
 
+#if defined(D3D12_SUPPORT)
+
 #include "util/defines.h"
 #include "util/logging.h"
 #include "util/options.h"
 #include "util/platform.h"
+#ifdef WIN32
 #include "graphics/dx12_image_renderer.h"
+#else
+#include "format/platform_types.h"
+#endif
 #include "format/format.h"
 
+#ifdef WIN32
 #include <comdef.h>
 #include <d3d12.h>
 #include <dxgi1_4.h>
+#endif
 #include <vector>
 #include <unordered_map>
 #include <map>
@@ -48,6 +56,7 @@ GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(graphics)
 GFXRECON_BEGIN_NAMESPACE(dx12)
 
+#ifdef WIN32
 typedef _com_ptr_t<_com_IIID<IDXGISwapChain3, &__uuidof(IDXGISwapChain3)>> IDXGISwapChain3ComPtr;
 
 typedef _com_ptr_t<_com_IIID<ID3D12DescriptorHeap, &__uuidof(ID3D12DescriptorHeap)>>     ID3D12DescriptorHeapComPtr;
@@ -78,6 +87,8 @@ typedef _com_ptr_t<_com_IIID<ID3D12GraphicsCommandList8, &__uuidof(ID3D12Graphic
     ID3D12GraphicsCommandList8ComPtr;
 typedef _com_ptr_t<_com_IIID<ID3D12GraphicsCommandList9, &__uuidof(ID3D12GraphicsCommandList9)>>
     ID3D12GraphicsCommandList9ComPtr;
+typedef _com_ptr_t<_com_IIID<ID3D12GraphicsCommandList10, &__uuidof(ID3D12GraphicsCommandList10)>>
+    ID3D12GraphicsCommandList10ComPtr;
 typedef _com_ptr_t<_com_IIID<ID3D12DeviceRemovedExtendedData1, &__uuidof(ID3D12DeviceRemovedExtendedData1)>>
     ID3D12DeviceRemovedExtendedData1ComPtr;
 typedef _com_ptr_t<
@@ -120,6 +131,7 @@ const static uint32_t kDrawCallArrayIndex       = 1;
 const static uint32_t kAfterDrawCallArrayIndex  = 2;
 
 uint32_t Dx12DumpResourcePosToArrayIndex(Dx12DumpResourcePos pos);
+#endif
 
 struct ActiveAdapterInfo
 {
@@ -155,6 +167,7 @@ struct ResourceStateInfo
 const D3D12_RANGE kZeroRange       = { 0, 0 };
 const double      kMemoryTolerance = 2.1;
 
+#ifdef WIN32
 UINT GetTexturePitch(UINT64 width);
 
 // Take a screenshot
@@ -291,6 +304,7 @@ bool IsMemoryAvailable(uint64_t requried_memory, IDXGIAdapter3* adapter, double
 // Get GPU memory usage by resource desc
 uint64_t GetResourceSizeInBytes(ID3D12Device* device, const D3D12_RESOURCE_DESC* desc);
 uint64_t GetResourceSizeInBytes(ID3D12Device8* device, const D3D12_RESOURCE_DESC1* desc);
+#endif
 
 bool IsSoftwareAdapter(const format::DxgiAdapterDesc& adapter_desc);
 
@@ -322,6 +336,7 @@ inline format::AdapterType ExtractAdapterType(uint32_t extra_info)
     return static_cast<format::AdapterType>((extra_info & kAdapterTypeMask));
 }
 
+#ifdef WIN32
 void RobustGetCopyableFootprint(ID3D12Device*                       device,
                                 ID3D12Resource*                     resource,
                                 const D3D12_RESOURCE_DESC*          pResourceDesc,
@@ -332,6 +347,7 @@ void RobustGetCopyableFootprint(ID3D12Device*                       device,
                                 UINT*                               pNumRows,
                                 UINT64*                             pRowSizeInBytes,
                                 UINT64*                             pTotalBytes);
+#endif
 
 bool IsFormatCompressed(DXGI_FORMAT format);
 
@@ -350,4 +366,6 @@ GFXRECON_END_NAMESPACE(dx12)
 GFXRECON_END_NAMESPACE(graphics)
 GFXRECON_END_NAMESPACE(gfxrecon)
 
+#endif // defined(D3D12_SUPPORT)
+
 #endif // GFXRECON_GRAPHICS_DX12_UTIL_H
