@@ -57,6 +57,10 @@
 #include <sys/system_properties.h>
 #endif
 
+#ifdef __linux__
+#include <sched.h>
+#endif
+
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(util)
 GFXRECON_BEGIN_NAMESPACE(platform)
@@ -260,6 +264,52 @@ inline int GetSystemLastErrorCode()
     return GetLastError();
 }
 
+inline std::string GetCpuAffinity()
+{
+    DWORD_PTR process_mask;
+    DWORD_PTR system_mask;
+    if (!GetProcessAffinityMask(GetCurrentProcess(), &process_mask, &system_mask))
+    {
+        return "";
+    }
+
+    DWORD_PTR mask = (process_mask & system_mask);
+
+    std::string affinity;
+    while (mask)
+    {
+        affinity += (mask & 1) ? "1" : "0";
+        mask >>= 1;
+    }
+
+    while (affinity.back() == '0')
+    {
+        affinity.pop_back();
+    }
+
+    return affinity;
+}
+
+inline bool SetCpuAffinity(const std::string& affinity)
+{
+    DWORD_PTR mask = 0;
+    for (unsigned i = 0; i < affinity.size(); i++)
+    {
+        if (affinity[i] == '1')
+        {
+            mask |= 1;
+        }
+        else if (affinity[i] != '0')
+        {
+            return false;
+        }
+
+        mask <<= 1;
+    }
+
+    return (SetProcessAffinityMask(GetCurrentProcess(), mask) != 0);
+}
+
 #else // !defined(WIN32)
 
 // Error value indicating string was truncated
@@ -568,6 +618,54 @@ inline int GetSystemLastErrorCode()
     return errno;
 }
 
+inline std::string GetCpuAffinity()
+{
+    std::string affinity;
+
+#ifdef __linux__
+    cpu_set_t mask;
+    if (sched_getaffinity(0, sizeof(mask), &mask))
+    {
+        return affinity;
+    }
+
+    for (unsigned i = 0; i < sizeof(mask) / CPU_ALLOC_SIZE(1); i++)
+    {
+        affinity += CPU_ISSET(i, &mask) ? "1" : "0";
+    }
+
+    while (affinity.back() == '0')
+    {
+        affinity.pop_back();
+    }
+#endif // __linux__
+
+    return affinity;
+}
+
+inline bool SetCpuAffinity(const std::string& affinity)
+{
+#ifdef __linux__
+    cpu_set_t mask;
+    CPU_ZERO(&mask);
+    for (unsigned i = 0; i < affinity.size(); i++)
+    {
+        if (affinity[i] == '1')
+        {
+            CPU_SET(i, &mask);
+        }
+        else if (affinity[i] != '0')
+        {
+            return false;
+        }
+    }
+
+    return (sched_setaffinity(0, sizeof(mask), &mask) == 0);
+#else // !__linux__
+    return false;
+#endif
+}
+
 #endif // WIN32
 
 inline size_t GetAlignedSize(size_t size, size_t align_to)
