@@ -10,6 +10,8 @@
 
 bool clean_gfxr_json(int depth, nlohmann::json::parse_event_t event, nlohmann::json& parsed)
 {
+    static bool skip_next_buffer = false;
+
     switch (event)
     {
         case nlohmann::json::parse_event_t::key:
@@ -23,10 +25,29 @@ bool clean_gfxr_json(int depth, nlohmann::json::parse_event_t event, nlohmann::j
                 return false;
             if (key == "\"pipelineCacheUUID\"")
                 return false;
+            if (key == "\"pipeline_cache_uuid\"")
+                return false;
             if (key == "\"ppData\"")
                 return false;
             if (key == "\"fd\"")
                 return false;
+            if (key == "\"app_name\"")
+                return false;
+            if (skip_next_buffer && key == "\"buffer\"")
+            {
+                skip_next_buffer = false;
+                return false;
+            }
+        }
+        break;
+        case nlohmann::json::parse_event_t::value:
+        {
+            auto value = to_string(parsed);
+            if (value == "\"VK_STRUCTURE_TYPE_IMPORT_ANDROID_HARDWARE_BUFFER_INFO_ANDROID\"" ||
+                value == "\"vkGetAndroidHardwareBufferPropertiesANDROID\"")
+            {
+                skip_next_buffer = true;
+            }
         }
         break;
         case nlohmann::json::parse_event_t::object_end:
@@ -34,8 +55,6 @@ bool clean_gfxr_json(int depth, nlohmann::json::parse_event_t event, nlohmann::j
                 return false;
             if (depth == 1 && parsed.contains("annotation"))
                 return false;
-            if (depth == 1 && parsed.contains("meta"))
-                return false;
             break;
         default:
             break;
@@ -67,7 +86,7 @@ struct Paths
     std::filesystem::path app_trimming_json_path;
     std::filesystem::path known_good_trimming_json_path;
 
-    void trimming_paths(char const* test_name, char const* known_gfxr_path, char const* trimming_frames)
+    void trimming_paths(char const* test_name, char const* trimming_frames)
     {
         // Trimming suffix is like "_frame_10" or "_frames_10_through_100"
         std::string s_trimming_frames = trimming_frames;
@@ -102,13 +121,8 @@ struct Paths
         capture_trimming_file += ".gfxr";
         capture_trimming_path.append(capture_trimming_file);
 
-        std::string s_known_gfxr_path        = known_gfxr_path;
-        index                                = s_known_gfxr_path.find(".gfxr");
-        std::string known_gfxr_trimming_path = s_known_gfxr_path.substr(0, index);
-        known_gfxr_trimming_path += trimming_suffix;
-        known_gfxr_trimming_path += ".gfxr";
         known_good_trimming_path.append("known_good");
-        known_good_trimming_path.append(known_gfxr_trimming_path);
+        known_good_trimming_path.append(capture_trimming_file);
 
         app_trimming_json_path = std::filesystem::path{ capture_trimming_path };
         app_trimming_json_path.replace_extension(".json");
@@ -117,7 +131,7 @@ struct Paths
         known_good_trimming_json_path.replace_extension(".json");
     }
 
-    Paths(char const* test_name, char const* known_gfxr_path, char const* trimming_frames)
+    Paths(char const* test_name, char const* trimming_frames)
     {
         working_directory = full_app_directory;
         working_directory.append("res");
@@ -138,7 +152,7 @@ struct Paths
         capture_path.append(gfxr_file_name);
 
         known_good_path.append("known_good");
-        known_good_path.append(known_gfxr_path);
+        known_good_path.append(gfxr_file_name);
 
         app_json_path = std::filesystem::path{ capture_path };
         app_json_path.replace_extension(".json");
@@ -148,7 +162,7 @@ struct Paths
 
         if (trimming_frames)
         {
-            trimming_paths(test_name, known_gfxr_path, trimming_frames);
+            trimming_paths(test_name, trimming_frames);
         }
     }
 };
@@ -225,7 +239,7 @@ int run_command(std::filesystem::path const& working_directory,
 
 void run_in_background(const char* test_name)
 {
-    Paths paths{ test_name, "", nullptr };
+    Paths paths{ test_name, nullptr };
     run_command(paths.working_directory, paths.full_executable_path, { test_name, "&" });
 }
 
@@ -266,11 +280,11 @@ void run_trimming_app(const Paths& paths, const char* test_name, char const* tri
     ASSERT_EQ(trimming_diff.size(), 0) << std::setw(4) << trimming_diff;
 }
 
-void verify_gfxr(const char* test_name, char const* known_gfxr_path, char const* trimming_frames)
+void verify_gfxr(const char* test_name, char const* trimming_frames)
 {
     EnvironmentVariables env_vars;
 
-    Paths paths{ test_name, known_gfxr_path, trimming_frames };
+    Paths paths{ test_name, trimming_frames };
     int   result;
 
     bool workind_directory_exists = std::filesystem::exists(paths.working_directory);
