@@ -58,6 +58,10 @@
 #include "parse_dump_resources_cli.h"
 #include "replay_pre_processing.h"
 
+// Includes for recapture
+#include "encode/vulkan_capture_manager.h"
+#include "recapture_vulkan_entry.h"
+
 #include <exception>
 #include <memory>
 #include <stdexcept>
@@ -66,7 +70,7 @@
 
 extern "C"
 {
-    __declspec(dllexport) extern const UINT D3D12SDKVersion = 615;
+    __declspec(dllexport) extern const UINT D3D12SDKVersion = 616;
 }
 extern "C"
 {
@@ -153,7 +157,7 @@ int main(int argc, const char** argv)
             // Select WSI context based on CLI
             std::string wsi_extension = GetWsiExtensionName(GetWsiPlatform(arg_parser));
             auto        application   = std::make_shared<gfxrecon::application::Application>(
-                kApplicationName, wsi_extension, file_processor.get());
+                kApplicationName, file_processor.get(), wsi_extension, nullptr);
 
             gfxrecon::decode::VulkanTrackedObjectInfoTable tracked_object_info_table;
             gfxrecon::decode::VulkanReplayOptions          vulkan_replay_options =
@@ -202,6 +206,23 @@ int main(int argc, const char** argv)
             gfxrecon::decode::VulkanReplayConsumer vulkan_replay_consumer(application, vulkan_replay_options);
             gfxrecon::decode::VulkanDecoder        vulkan_decoder;
 
+            if (vulkan_replay_options.capture)
+            {
+                gfxrecon::vulkan_recapture::RecaptureVulkanEntry::InitSingleton();
+
+                // Set replay to use the GetInstanceProcAddr function from RecaptureVulkanEntry so that replay first
+                // calls into the capture layer instead of directly into the loader and Vulkan runtime.
+                vulkan_replay_consumer.SetGetInstanceProcAddrOverride(gfxrecon::vulkan_recapture::GetInstanceProcAddr);
+
+                // Set the capture manager's instance and device creation callbacks.
+                gfxrecon::encode::VulkanCaptureManager::SetLayerFuncs(
+                    gfxrecon::vulkan_recapture::dispatch_CreateInstance,
+                    gfxrecon::vulkan_recapture::dispatch_CreateDevice);
+
+                // Logger is already initialized by replay, so inform capture manager not to initialize it again.
+                gfxrecon::encode::CommonCaptureManager::SetInitializeLog(false);
+            }
+
             ApiReplayOptions  api_replay_options;
             ApiReplayConsumer api_replay_consumer;
             api_replay_options.vk_replay_options   = &vulkan_replay_options;
@@ -344,6 +365,11 @@ int main(int argc, const char** argv)
             {
                 GFXRECON_WRITE_CONSOLE("File did not contain any frames");
             }
+
+            if (vulkan_replay_options.capture)
+            {
+                gfxrecon::vulkan_recapture::RecaptureVulkanEntry::DestroySingleton();
+            }
         }
     }
     catch (const std::runtime_error& error)
