@@ -38,6 +38,7 @@
 #include "decode/vulkan_resource_tracking_consumer.h"
 #include "decode/vulkan_tracked_object_info_table.h"
 #include "generated/generated_vulkan_decoder.h"
+#include "format/format.h"
 
 #if ENABLE_OPENXR_SUPPORT
 #include "generated/generated_openxr_decoder.h"
@@ -71,6 +72,7 @@ const char kHelpShortOption[]                    = "-h";
 const char kHelpLongOption[]                     = "--help";
 const char kVersionOption[]                      = "--version";
 const char kLogLevelArgument[]                   = "--log-level";
+const char kLogTimestampsOption[]                = "--log-timestamps";
 const char kDebugMessageSeverityArgument[]       = "--debug-messenger-level";
 const char kLogFileArgument[]                    = "--log-file";
 const char kLogDebugView[]                       = "--log-debugview";
@@ -80,6 +82,7 @@ const char kOverrideGpuArgument[]                = "--gpu";
 const char kOverrideGpuGroupArgument[]           = "--gpu-group";
 const char kPausedOption[]                       = "--paused";
 const char kPauseFrameArgument[]                 = "--pause-frame";
+const char kCaptureOption[]                      = "--capture";
 const char kSkipFailedAllocationShortOption[]    = "--sfa";
 const char kSkipFailedAllocationLongOption[]     = "--skip-failed-allocations";
 const char kDiscardCachedPsosShortOption[]       = "--dcp";
@@ -121,6 +124,7 @@ const char kQuitAfterFrameArgument[]             = "--quit-after-frame";
 const char kFlushMeasurementRangeOption[]        = "--flush-measurement-range";
 const char kFlushInsideMeasurementRangeOption[]  = "--flush-inside-measurement-range";
 const char kSwapchainOption[]                    = "--swapchain";
+const char kPresentModeOption[]                  = "--present-mode";
 const char kEnableUseCapturedSwapchainIndices[] =
     "--use-captured-swapchain-indices"; // The same: util::SwapchainOption::kCaptured
 const char kVirtualSwapchainSkipBlitShortOption[] = "--vssb";
@@ -153,21 +157,22 @@ const char kDxAgsMarkRenderPasses[]       = "--dx12-ags-inject-markers";
 const char kBatchingMemoryUsageArgument[] = "--batching-memory-usage";
 #endif
 
-const char kDumpResourcesArgument[]                 = "--dump-resources";
-const char kDumpResourcesBeforeDrawOption[]         = "--dump-resources-before-draw";
-const char kDumpResourcesImageFormat[]              = "--dump-resources-image-format";
-const char kDumpResourcesScaleArgument[]            = "--dump-resources-scale";
-const char kDumpResourcesDepth[]                    = "--dump-resources-dump-depth-attachment";
-const char kDumpResourcesDirArgument[]              = "--dump-resources-dir";
-const char kDumpResourcesModifiableStateOnly[]      = "--dump-resources-modifiable-state-only";
-const char kDumpResourcesColorAttIdxArg[]           = "--dump-resources-dump-color-attachment-index";
-const char kDumpResourcesDumpVertexIndexBuffers[]   = "--dump-resources-dump-vertex-index-buffers";
-const char kDumpResourcesJsonPerCommand[]           = "--dump-resources-json-output-per-command";
-const char kDumpResourcesDumpImmutableResources[]   = "--dump-resources-dump-immutable-resources";
-const char kDumpResourcesDumpImageSubresources[]    = "--dump-resources-dump-all-image-subresources";
-const char kDumpResourcesDumpRawImages[]            = "--dump-resources-dump-raw-images";
-const char kDumpResourcesDumpSeparateAlpha[]        = "--dump-resources-dump-separate-alpha";
-const char kDumpResourcesDumpUnusedVertexBindings[] = "--dump-resources-dump-unused-vertex-bindigs";
+const char kDumpResourcesArgument[]                    = "--dump-resources";
+const char kDumpResourcesBeforeDrawOption[]            = "--dump-resources-before-draw";
+const char kDumpResourcesImageFormat[]                 = "--dump-resources-image-format";
+const char kDumpResourcesScaleArgument[]               = "--dump-resources-scale";
+const char kDumpResourcesDepth[]                       = "--dump-resources-dump-depth-attachment";
+const char kDumpResourcesDirArgument[]                 = "--dump-resources-dir";
+const char kDumpResourcesModifiableStateOnly[]         = "--dump-resources-modifiable-state-only";
+const char kDumpResourcesColorAttIdxArg[]              = "--dump-resources-dump-color-attachment-index";
+const char kDumpResourcesDumpVertexIndexBuffers[]      = "--dump-resources-dump-vertex-index-buffers";
+const char kDumpResourcesJsonPerCommand[]              = "--dump-resources-json-output-per-command";
+const char kDumpResourcesDumpImmutableResources[]      = "--dump-resources-dump-immutable-resources";
+const char kDumpResourcesDumpImageSubresources[]       = "--dump-resources-dump-all-image-subresources";
+const char kDumpResourcesDumpRawImages[]               = "--dump-resources-dump-raw-images";
+const char kDumpResourcesDumpSeparateAlpha[]           = "--dump-resources-dump-separate-alpha";
+const char kDumpResourcesDumpUnusedVertexBindings[]    = "--dump-resources-dump-unused-vertex-bindigs";
+const char kDumpResourcesBinaryFileCompressionMethod[] = "--dump-resources-binary-file-compression-type";
 
 enum class WsiPlatform
 {
@@ -199,9 +204,20 @@ const char kSwapchainVirtual[]   = "virtual";
 const char kSwapchainCaptured[]  = "captured";
 const char kSwapchainOffscreen[] = "offscreen";
 
+const char kPresentModeCapture[]     = "capture";
+const char kPresentModeImmediate[]   = "immediate";
+const char kPresentModeMailbox[]     = "mailbox";
+const char kPresentModeFifo[]        = "fifo";
+const char kPresentModeFifoRelaxed[] = "fifo_relaxed";
+
 const char kScreenshotFormatBmp[] = "bmp";
 const char kScreenshotFormatPng[] = "png";
 
+const char kCompressionTypeNone[] = "none";
+const char kCompressionTypeLz4[]  = "lz4";
+const char kCompressionTypeZlib[] = "zlib";
+const char kCompressionTypeZstd[] = "zstd";
+
 #if defined(__ANDROID__)
 const char kDefaultScreenshotDir[]    = "/sdcard";
 const char kDefaultDumpResourcesDir[] = "/sdcard";
@@ -535,6 +551,7 @@ static void GetLogSettings(const gfxrecon::util::ArgumentParser& arg_parser,
 
     // Update settings
     log_settings.min_severity              = log_level;
+    log_settings.output_timestamps         = arg_parser.IsOptionSet(kLogTimestampsOption);
     log_settings.file_name                 = arg_parser.GetArgumentValue(kLogFileArgument);
     log_settings.output_to_os_debug_string = arg_parser.IsOptionSet(kLogDebugView);
 }
@@ -600,6 +617,39 @@ static gfxrecon::util::ScreenshotFormat GetDumpresourcesImageFormat(const gfxrec
     return format;
 }
 
+static gfxrecon::format::CompressionType
+GetDumpResourcesCompressionType(const gfxrecon::util::ArgumentParser& arg_parser)
+{
+    const auto& value = arg_parser.GetArgumentValue(kDumpResourcesBinaryFileCompressionMethod);
+
+    gfxrecon::format::CompressionType type = gfxrecon::format::CompressionType::kNone;
+    if (!value.empty())
+    {
+        if (gfxrecon::util::platform::StringCompareNoCase(kCompressionTypeNone, value.c_str()) == 0)
+        {
+            type = gfxrecon::format::CompressionType::kNone;
+        }
+        else if (gfxrecon::util::platform::StringCompareNoCase(kCompressionTypeLz4, value.c_str()) == 0)
+        {
+            type = gfxrecon::format::CompressionType::kLz4;
+        }
+        else if (gfxrecon::util::platform::StringCompareNoCase(kCompressionTypeZlib, value.c_str()) == 0)
+        {
+            type = gfxrecon::format::CompressionType::kZlib;
+        }
+        else if (gfxrecon::util::platform::StringCompareNoCase(kCompressionTypeZstd, value.c_str()) == 0)
+        {
+            type = gfxrecon::format::CompressionType::kZstd;
+        }
+        else
+        {
+            GFXRECON_LOG_ERROR("Unrecognized compression method \"%s\"", value.c_str());
+        }
+    }
+
+    return type;
+}
+
 static std::string GetScreenshotDir(const gfxrecon::util::ArgumentParser& arg_parser)
 {
     const auto& value = arg_parser.GetArgumentValue(kScreenshotDirArgument);
@@ -1033,6 +1083,11 @@ GetVulkanReplayOptions(const gfxrecon::util::ArgumentParser&           arg_parse
     gfxrecon::decode::VulkanReplayOptions replay_options;
     GetReplayOptions(replay_options, arg_parser, filename);
 
+    if (arg_parser.IsOptionSet(kCaptureOption))
+    {
+        replay_options.capture = true;
+    }
+
     const auto& override_gpu_group = arg_parser.GetArgumentValue(kOverrideGpuGroupArgument);
     if (!override_gpu_group.empty())
     {
@@ -1088,6 +1143,32 @@ GetVulkanReplayOptions(const gfxrecon::util::ArgumentParser&           arg_parse
         }
     }
 
+    auto present_mode_option = arg_parser.GetArgumentValue(kPresentModeOption);
+    if (gfxrecon::util::platform::StringCompareNoCase(kPresentModeCapture, present_mode_option.c_str()) == 0)
+    {
+        replay_options.present_mode_option = gfxrecon::util::PresentModeOption::kCapture;
+    }
+    else if (gfxrecon::util::platform::StringCompareNoCase(kPresentModeImmediate, present_mode_option.c_str()) == 0)
+    {
+        replay_options.present_mode_option = gfxrecon::util::PresentModeOption::kImmediate;
+    }
+    else if (gfxrecon::util::platform::StringCompareNoCase(kPresentModeMailbox, present_mode_option.c_str()) == 0)
+    {
+        replay_options.present_mode_option = gfxrecon::util::PresentModeOption::kMailbox;
+    }
+    else if (gfxrecon::util::platform::StringCompareNoCase(kPresentModeFifo, present_mode_option.c_str()) == 0)
+    {
+        replay_options.present_mode_option = gfxrecon::util::PresentModeOption::kFifo;
+    }
+    else if (gfxrecon::util::platform::StringCompareNoCase(kPresentModeFifoRelaxed, present_mode_option.c_str()) == 0)
+    {
+        replay_options.present_mode_option = gfxrecon::util::PresentModeOption::kFifoRelaxed;
+    }
+    else if (!present_mode_option.empty())
+    {
+        GFXRECON_LOG_WARNING("Ignoring unrecognized \"--present-mode\" option: %s", present_mode_option.c_str());
+    }
+
     if (arg_parser.IsOptionSet(kColorspaceFallback))
     {
         replay_options.use_colorspace_fallback = true;
@@ -1252,6 +1333,7 @@ GetVulkanReplayOptions(const gfxrecon::util::ArgumentParser&           arg_parse
     replay_options.dump_resources_dump_separate_alpha = arg_parser.IsOptionSet(kDumpResourcesDumpSeparateAlpha);
     replay_options.dump_resources_dump_unused_vertex_bindings =
         arg_parser.IsOptionSet(kDumpResourcesDumpUnusedVertexBindings);
+    replay_options.dump_resources_binary_file_compression_type = GetDumpResourcesCompressionType(arg_parser);
 
     std::string dr_color_att_idx = arg_parser.GetArgumentValue(kDumpResourcesColorAttIdxArg);
     if (!dr_color_att_idx.empty())
